"""Tests for TOPFARM
"""
import warnings
import numpy as np
from topfarm.topfarm import TopFarm
from topfarm.cost_models.dummy import DummyCost
import unittest


class Test(unittest.TestCase):  # unittest version

    def test_optimize_4tb(self):
        """Optimize 4-turbine layout and check final positions

            The farm boundaries and min spacing are chosen such that the desired
            turbine positions are not within the boundaries or constraints.
        """

        # test options
        dec_prec = 4  # decimal precision for comparison

        # given
        boundary = [(0, 0), (6, 0), (6, -10), (0, -10)]  # turbine boundaries
        initial = [[6, 0], [6, -8], [1, 1], [-1, -8]]  # initial turbine layouts
        desired = [[3, -3], [7, -7], [4, -3], [3, -7]]  # desired turbine layouts
        optimal = np.array([[2.5, -3], [6, -7], [4.5, -3], [3, -7]])  # optimal turbine layout
        min_spacing = 2  # min distance between turbines

        # when
        tf = TopFarm(initial, DummyCost(desired), min_spacing,
                     boundary=boundary)
        with warnings.catch_warnings():  # suppress OpenMDAO/SLSQP warnings
            warnings.simplefilter('ignore')
            tf.optimize()
        tb_pos = tf.turbine_positions

        # then
        tol = 1e-6
        self.assertGreater(sum((tb_pos[2] - tb_pos[0])**2), 2**2 - tol)  # check min spacing
        self.assertLess(tb_pos[1][0], 6 + tol)  # check within border
        np.testing.assert_array_almost_equal(tb_pos, optimal, dec_prec)


if __name__ == "__main__":
    unittest.main()
